/**
 * Copyright (c) 2019 AnyGame. All rights reserved.
 * 事件派发
 * @class clientEvent
 * @static
 */

export default class clientEvent {
    private static handlerList = {};
    /**
     * 监听事件
     * @param {string} eventName 事件名称
     * @param {function} handler 监听函数
     * @param {object} target 监听目标
     * @method on
     */
    static on(eventName, handler, target) {
        if (typeof handler !== 'function') {
            console.error(`[clientEvent] handler must be function when register ${eventName}`);
            return;
        }

        if (!clientEvent.handlerList[eventName]) {
            clientEvent.handlerList[eventName] = [];
        }
        const list = clientEvent.handlerList[eventName];
        let alreadyRegister = false;
        const { length } = list;
        for (let i = 0; i < length; i ++) {
            const handlerInList = list[i].handler;
            const targetInList = list[i].target;
            if (handler === handlerInList && target === targetInList) {
                alreadyRegister = true;
                console.warn(`[clientEvent] ${eventName} already register with same hanlder and target.`);
                break;
            }
        }

        if (!alreadyRegister) {
            list.push({
                handler,
                target,
            });
        }
    }
    /**
     * 取消监听
     * @param {string} eventName 监听事件
     * @param {function} handler 监听函数
     * @param {object} target 监听目标
     * @method off
     */
    static off(eventName, handler, target) { 
        const list = clientEvent.handlerList[eventName];
        if (list) {
            for (let i = 0; i < list.length; i ++) {
                const handlerInList = list[i].handler;
                const targetInList = list[i].target;
                if (handler === handlerInList && target === targetInList) {
                    list.splice(i, 1);
                    break;
                }
            }
        }
    }
    /**
     * 分发事件
     * @param {string} eventName 分发事件名
     * @param  {...any} params 分发事件参数
     * @method dispatchEvent
     */
    static dispatchEvent(eventName, ...params) {
        const list = clientEvent.handlerList[eventName];
        if (!list || list.length === 0) {
            console.warn(`[clientEvent] please retister ${eventName} first.`);
            return;
        }

        for (const data of list) {
            const { handler, target } = data;
            handler.apply(target, params);
        }
    }
};