// Learn TypeScript:
//  - https://docs.cocos.com/creator/manual/en/scripting/typescript.html
// Learn Attribute:
//  - https://docs.cocos.com/creator/manual/en/scripting/reference/attributes.html
// Learn life-cycle callbacks:
//  - https://docs.cocos.com/creator/manual/en/scripting/life-cycle-callbacks.html

import clientEvent from "./clientEvent";

const {ccclass, property} = cc._decorator;
@ccclass
export default class NewClass extends cc.Component {

    @property(cc.Node)
    target: cc.Node = null;
    rigidBody: cc.RigidBody;
    motor: cc.MotorJoint;
    collider: cc.PhysicsPolygonCollider;
    startPoint: cc.Vec2;
    tween: any;
    points: cc.Vec2;

    // LIFE-CYCLE CALLBACKS:
    onPreSolve(contact, selfCollider, otherCollider) {
        // console.error(contact);
        const worldManifold = contact.getWorldManifold();
        this.points = worldManifold.normal;
        if (otherCollider.node.name.includes('staticGear')) {
            this.scheduleOnce(() => {
                this.node.position = cc.v3(this.points.x, this.points.y, 0);
                this.rigidBody.angularVelocity = - otherCollider.node.getComponent(cc.RigidBody).angularVelocity;
            },0);
        }
        }

        onPostSolve(contact, selfCollider, otherCollider) {
            this.rigidBody.angularVelocity = 0;
        }

    onLoad () {
        this.initEvent();
        this.rigidBody = this.node.getComponent(cc.RigidBody);
    }

    onEnable() {
    }

    initEvent() {
        this.node.on(cc.Node.EventType.TOUCH_START, this._touchStartEvent, this);
        this.node.on(cc.Node.EventType.TOUCH_MOVE, this._touchMoveEvent, this);
        this.node.on(cc.Node.EventType.TOUCH_END, this._touchEndEvent, this);
        this.node.on(cc.Node.EventType.TOUCH_CANCEL, this._touchCancelEvent, this);
    }

    _touchStartEvent(event: cc.Event.EventTouch) {
        this.startPoint = event.getStartLocation();
    }
    _touchMoveEvent(event: cc.Event.EventTouch) {
        // this.points.push(event.getLocation());
        // if (this.points.length > 20) {
        //         this.points.shift();
        //     }
        //     const averagePoint = new cc.Vec2(0, 0);
        //     this.points.forEach((point) => {
        //             averagePoint.x += point.x;
        //             averagePoint.y += point.y;
        //         })
        //         averagePoint.multiplyScalar(1 / this.points.length);
                
        //         const delta = new cc.Vec2(event.getLocation().x - averagePoint.x, event.getLocation().y - averagePoint.y);
        //         let dir = delta;
        let dir = cc.v2(event.getLocation().x - this.startPoint.x, event.getLocation().y - this.startPoint.y);
        // clientEvent.dispatchEvent('move', dir);
        this.move(dir);
    }

    _touchEndEvent(event: cc.Event.EventTouch) {
        // this.points = [];
        // clientEvent.dispatchEvent('moveEnd');
    }

    _touchCancelEvent(event: cc.Event.EventTouch) {
        // clientEvent.dispatchEvent('moveEnd');
        this.moveEnd();
    }

    move(dir) {
        this.node.position = dir;
        // this.rigidBody.awake = true;
        cc.director.getPhysicsManager().enabled = false;
    }
    moveEnd() {
        cc.director.getPhysicsManager().enabled = true;

    }
    start () {

    }

    update (dt) {
    }

}
